@@ -17,10 +17,13 @@ from asusrouter import (
     AsusRouterServiceError,
     AsusRouterValueError,
     Connection,
+    FilterDevice,
     Monitor,
 )
 from asusrouter.const import (
+    AIMESH,
     AR_DEVICE_IDENTITY,
+    AR_FIRMWARE_CHECK_COMMAND,
     AR_HOOK_DEVICES,
     AR_KEY_AURARGB,
     AR_KEY_CPU,
@@ -36,7 +39,6 @@ from asusrouter.const import (
     AR_KEY_SERVICE_COMMAND,
     AR_KEY_SERVICE_MODIFY,
     AR_KEY_SERVICE_REPLY,
-    AR_KEY_VPN_CLIENT,
     AR_KEY_WAN,
     AR_LEDG_MODE,
     AR_MAP_PARENTAL_CONTROL,
@@ -44,6 +46,8 @@ from asusrouter.const import (
     AR_SERVICE_COMMAND,
     AR_SERVICE_CONTROL,
     AR_SERVICE_DROP_CONNECTION,
+    CLIENTS,
+    CONNECTION_TYPE,
     DATA_BY_CORE,
     DATA_TOTAL,
     DATA_USAGE,
@@ -55,7 +59,9 @@ from asusrouter.const import (
     ERROR_SERVICE,
     ERROR_SERVICE_UNKNOWN,
     ERROR_VALUE,
+    GUEST,
     INTERFACE_TYPE,
+    IP,
     KEY_ACTION_MODE,
     KEY_CPU,
     KEY_HOOK,
@@ -65,17 +71,23 @@ from asusrouter.const import (
     KEY_TEMPERATURE,
     KEY_VPN,
     KEY_WAN,
+    MAC,
     MONITOR_MAIN,
     MSG_ERROR,
     MSG_INFO,
     MSG_SUCCESS,
+    NODE,
     NVRAM_LIST,
     NVRAM_TEMPLATE,
+    ONBOARDING,
     PARAM_COLOR,
     PARAM_COUNT,
     PARAM_MODE,
     PORT_TYPE,
+    RSSI,
+    TRACK_SERVICES_LED,
 )
+from asusrouter.dataclass import AiMeshDevice
 from asusrouter.util import calculators, compilers, converters, parsers
 
 _LOGGER = logging.getLogger(__name__)
@@ -110,6 +122,10 @@ class AsusRouter:
         self._device_ports: dict[str, str] | None = None
         self._device_boottime: datetime | None = None
 
+        self.monitor: dict[str, Monitor] = {
+            ONBOARDING: Monitor(),
+        }
+
         self._monitor_main: Monitor = Monitor()
         self._monitor_nvram: Monitor = Monitor()
         self._monitor_misc: Monitor = Monitor()
@@ -219,6 +235,14 @@ class AsusRouter:
                     ERROR_IDENTITY.format(self._host, str(ex))
                 )
 
+        # Firmware
+        identity["firmware"] = parsers.firmware_string(
+            f"{identity['fw_major']}.{identity['fw_minor']}.{identity['fw_build']}"
+        )
+        identity.pop("fw_major")
+        identity.pop("fw_minor")
+        identity.pop("fw_build")
+
         # Check by page
         identity["sysinfo"] = await self.async_check_endpoint(
             endpoint=AR_PATH["sysinfo"]
@@ -226,6 +250,10 @@ class AsusRouter:
         identity["onboarding"] = await self.async_check_endpoint(
             endpoint=AR_PATH["onboarding"]
         )
+        self._monitor_onboarding = identity["onboarding"]
+        identity["update_networkmapd"] = await self.async_check_endpoint(
+            endpoint=AR_PATH["networkmap"]
+        )
 
         # Check RGBG / AURA
         try:
@@ -257,7 +285,10 @@ class AsusRouter:
     ### DATA PROCESSING -->
 
     async def async_command(
-        self, commands: dict[str, str], action_mode: str = DEFAULT_ACTION_MODE
+        self,
+        commands: dict[str, str] | None = None,
+        action_mode: str = DEFAULT_ACTION_MODE,
+        apply_to: str = AR_PATH["command"],
     ) -> dict[str, Any]:
         """Command device to run a service or set parameter"""
 
@@ -270,14 +301,14 @@ class AsusRouter:
         request: dict = {
             KEY_ACTION_MODE: action_mode,
         }
-        for command in commands:
-            request[command] = commands[command]
+        if commands is not None:
+            for command in commands:
+                request[command] = commands[command]
 
         try:
             result = await self.connection.async_run_command(
-                command=str(request), endpoint=AR_PATH["command"]
+                command=str(request), endpoint=apply_to
             )
-            _LOGGER.debug("{}: {}".format(MSG_SUCCESS["command"], request))
         except Exception as ex:
             raise ex
 
@@ -302,7 +333,6 @@ class AsusRouter:
             result = await self.connection.async_run_command(
                 command="{}={}".format(KEY_HOOK, hook)
             )
-            _LOGGER.debug("{}: {}".format(MSG_SUCCESS["hook"], hook))
         except Exception as ex:
             raise ex
 
@@ -328,7 +358,6 @@ class AsusRouter:
 
         try:
             result = await self.connection.async_run_command(command="", endpoint=page)
-            _LOGGER.debug(MSG_SUCCESS["load"].format(page))
         except AsusRouter404 as ex:
             raise ex
         except Exception as ex:
@@ -573,9 +602,11 @@ class AsusRouter:
                         elif timestamp - _timestamp < 0:
                             # Leave the same boot time, since we don't know the new correct time
                             monitor_misc["BOOTTIME"] = self._monitor_misc["BOOTTIME"]
+                        # Boot time changed -> there was reboot
                         else:
                             monitor_misc["BOOTTIME"]["timestamp"] = timestamp
                             monitor_misc["BOOTTIME"]["ISO"] = time.isoformat()
+                            await self.async_handle_reboot()
                     else:
                         monitor_misc["BOOTTIME"]["timestamp"] = timestamp
                         monitor_misc["BOOTTIME"]["ISO"] = time.isoformat()
@@ -591,6 +622,62 @@ class AsusRouter:
 
         return
 
+    async def async_monitor_onboarding(self) -> None:
+        """Get the onboarding state"""
+
+        # Check whether to run
+        if not self.async_monitor_should_run(ONBOARDING):
+            return
+
+        try:
+            # Start
+            self.monitor[ONBOARDING].start()
+            monitor = Monitor()
+            # Hook data
+            raw = await self.async_load(AR_PATH["onboarding"])
+            # Reset time
+            monitor.reset()
+
+            # Process data
+
+            # AiMesh nodes state
+            aimesh = dict()
+            data = raw["get_cfg_clientlist"][0]
+            for device in data:
+                node = parsers.aimesh_node(device)
+                aimesh[node.mac] = node
+            monitor[AIMESH] = aimesh
+
+            # Client list
+            clients = dict()
+            data = raw["get_allclientlist"][0]
+            for node in data:
+                for connection in data[node]:
+                    for mac in data[node][connection]:
+                        convert = converters.onboarding_connection(connection)
+                        description = {
+                            CONNECTION_TYPE: convert[CONNECTION_TYPE],
+                            GUEST: convert[GUEST],
+                            IP: converters.none_or_str(
+                                data[node][connection][mac].get(IP, None)
+                            ),
+                            MAC: mac,
+                            NODE: node,
+                            RSSI: data[node][connection][mac].get(RSSI, None),
+                        }
+                        clients[mac] = description
+            monitor[CLIENTS] = clients
+
+            # Finish and save data
+            monitor.finish()
+            self.monitor[ONBOARDING] = monitor
+
+        except AsusRouterError as ex:
+            self.monitor[ONBOARDING].drop()
+            raise ex
+
+        return
+
     async def async_monitor_devices(self) -> None:
         """Monitor connected devices"""
 
@@ -602,16 +689,27 @@ class AsusRouter:
             self._monitor_devices.start()
             monitor_devices = Monitor()
 
+            # Update device list (needed for older devices)
+            await self.async_check_endpoint(AR_PATH["devices_update"])
+            if self._identity.update_networkmapd:
+                await self.async_check_endpoint(AR_PATH["networkmap"])
             data = await self.async_hook(AR_HOOK_DEVICES)
 
             monitor_devices.reset()
 
+            onboarding = {}
+            if self._identity.onboarding:
+                await self.async_monitor_onboarding()
+                onboarding = self.monitor[ONBOARDING][CLIENTS]
+
             # Search for new data
             if AR_KEY_DEVICES in data:
                 data = data[AR_KEY_DEVICES]
                 for mac in data:
                     if converters.is_mac_address(mac):
-                        monitor_devices[mac] = parsers.connected_device(data[mac])
+                        monitor_devices[mac] = compilers.connected_device(
+                            parsers.connected_device(data[mac]), onboarding.get(mac)
+                        )
             # Or keep last data
             elif self._monitor_devices.ready:
                 monitor_devices = self._monitor_devices
@@ -626,11 +724,57 @@ class AsusRouter:
 
     ### <-- MONITORS
 
+    ### TECHNICAL -->
+
+    async def async_monitor_available(self, monitor: str) -> bool:
+        """Check whether monitor is available"""
+
+        # Monitor does not exist
+        if not monitor in self.monitor:
+            return False
+
+        # Monitor is disabled
+        if self.monitor[monitor].enabled != True:
+            return False
+
+        return True
+
+    async def async_monitor_cached(self, monitor: str, value: Any) -> bool:
+        """Check whether monitor has cached value"""
+
+        now = datetime.utcnow()
+        if (
+            not self.monitor[monitor].ready
+            or not value in self.monitor[monitor]
+            or self._cache_time < (now - self.monitor[monitor].time).total_seconds()
+        ):
+            return False
+
+        return True
+
+    async def async_monitor_should_run(self, monitor: str) -> bool:
+        """Check whether monitor should be run"""
+
+        # Monitor is not available
+        if await self.async_monitor_available(monitor) == False:
+            return False
+
+        # Monitor is already running - wait to complete
+        if self.monitor[monitor].active:
+            while self.monitor[monitor].active:
+                await asyncio.sleep(DEFAULT_SLEEP_TIME)
+            return False
+
+        return True
+
+    ### <-- TECHNICAL
+
     async def async_handle_error(self) -> None:
         """Actions to be taken on connection errors"""
 
-        # Most of errors come from device reboots. Handle it
-        await self.async_handle_reboot()
+        # Clear main monitor to prevent calculation errors in it
+        if self._monitor_main.ready:
+            self._monitor_main = Monitor()
 
         # Clear error
         await self.connection.async_reset_error()
@@ -640,9 +784,7 @@ class AsusRouter:
     async def async_handle_reboot(self) -> None:
         """Actions to be taken on reboot"""
 
-        # Clear main monitor to prevent calculation errors in it
-        if self._monitor_main.ready:
-            self._monitor_main = Monitor()
+        await self.async_keep_state_led()
 
         return
 
@@ -677,6 +819,26 @@ class AsusRouter:
 
     ### RETURN DATA -->
 
+    async def async_get_aimesh(self, use_cache: bool = True) -> dict[str, AiMeshDevice]:
+        """Return AiMesh device list"""
+
+        monitor = ONBOARDING
+        result = dict()
+
+        # Monitor is not available
+        if not await self.async_monitor_available(monitor):
+            return result
+
+        # Value is not cached or cache is disabled
+        if not await self.async_monitor_cached(monitor, AIMESH) or use_cache == False:
+            await self.async_monitor_onboarding()
+
+        # Process data
+        result = self.monitor[monitor][AIMESH]
+
+        # Return data
+        return result
+
     async def async_get_cpu(self, use_cache: bool = True) -> dict[str, float]:
         """Return CPU usage"""
 
@@ -756,6 +918,42 @@ class AsusRouter:
 
         return self._monitor_devices.copy()
 
+    async def async_get_gwlan(self) -> dict[str, Any]:
+        """Get state of guest WLAN by id"""
+
+        ids = await self.async_get_gwlan_ids()
+
+        # NVRAM values to check
+        nvram = list()
+        for id in ids:
+            for value in NVRAM_TEMPLATE["GWLAN"]:
+                nvram.append(value.value.format(id))
+
+        try:
+            data = await self.async_hook(compilers.nvram(nvram))
+        except AsusRouterError as ex:
+            raise ex
+        for id in ids:
+            for value in NVRAM_TEMPLATE["GWLAN"]:
+                data[value.value.format(id)] = value.method(
+                    data[value.value.format(id)]
+                )
+
+        return data
+
+    async def async_get_gwlan_ids(self) -> list[int]:
+        """Return list of guest WLAN ids"""
+
+        ids = list()
+
+        wlans = await self.async_get_wlan_ids()
+
+        for wlan in wlans:
+            for i in range(1, 4):
+                ids.append(f"{wlan}.{i}")
+
+        return ids
+
     async def async_get_network(
         self, use_cache: bool = True
     ) -> dict[str, (int | float)]:
@@ -807,6 +1005,21 @@ class AsusRouter:
 
         return result
 
+    ### PARENTAL CONTROL -->
+    async def async_apply_parental_control_rules(
+        self,
+        rules: dict[str, FilterDevice],
+    ) -> bool:
+        """Apply parental control rules"""
+
+        request = compilers.parental_control(rules)
+        request["action_mode"] = "apply"
+
+        return await self.async_service_run(
+            service="restart_firewall",
+            arguments=request,
+        )
+
     async def async_get_parental_control(self) -> dict[str, Any]:
         """Return parental control status"""
 
@@ -821,14 +1034,75 @@ class AsusRouter:
 
         return data
 
-    async def async_get_parental_control_device(self, mac: str) -> dict[str, Any]:
-        """Return parental control status of a device by its MAC"""
+    async def async_get_parental_control_rules(
+        self,
+        macs: str | list[str] | None = None,
+    ) -> dict[str, FilterDevice]:
+        """Return parental control rules"""
 
-        devices = await self.async_get_parental_control()
-        if mac in devices["list"]:
-            return devices["list"][mac]
-        else:
-            return {}
+        pc: dict = await self.async_get_parental_control()
+        rules = pc.get("list", dict())
+
+        # Return the full list if no device specified
+        if macs is None:
+            return rules
+
+        # Return only devices of interest
+        result = dict()
+        macs = converters.as_list(macs)
+        for mac in macs:
+            if mac in rules:
+                result[mac] = rules[mac]
+        return result
+
+    async def async_remove_parental_control_rules(
+        self,
+        macs: str | list[str] | None = None,
+        rules: FilterDevice | list[FilterDevice] = None,
+        apply: bool = True,
+    ) -> dict[str, FilterDevice]:
+        """Remove parental control rules"""
+
+        macs = list() if macs is None else converters.as_list(macs)
+        rules = list() if rules is None else converters.as_list(rules)
+
+        # Get current rules
+        cr: dict = await self.async_get_parental_control_rules()
+
+        # Remove old rules for these MACs
+        for mac in macs:
+            if mac in cr:
+                cr.pop(mac)
+        for rule in rules:
+            if rule.mac in cr:
+                cr.pop(rule.mac)
+
+        # Apply new rules
+        if apply:
+            await self.async_apply_parental_control_rules(cr)
+
+        # Return the new rules
+        return cr
+
+    async def async_set_parental_control_rules(
+        self,
+        rules: FilterDevice | list[FilterDevice],
+    ) -> bool:
+        """Set parental control rules"""
+
+        rules = converters.as_list(rules)
+
+        # Remove old rules for these MACs and get the rest of the list
+        cr = await self.async_remove_parental_control_rules(rules, apply=False)
+
+        # Add new rules
+        for rule in rules:
+            cr[rule.mac] = rule
+
+        # Apply new rules
+        return await self.async_apply_parental_control_rules(cr)
+
+    ### <-- PARENTAL CONTROL
 
     async def async_get_ports(
         self, use_cache: bool = True
@@ -873,18 +1147,6 @@ class AsusRouter:
 
         return result
 
-    async def async_get_ram_labels(self) -> list[str]:
-        """Return list of CPU cores"""
-
-        if not self._monitor_main.ready:
-            await self.async_monitor_main()
-
-        result = list()
-        for value in self._monitor_main[KEY_RAM]:
-            result.append(value)
-
-        return result
-
     async def async_get_sysinfo(self, use_cache: bool = True) -> dict[str, Any]:
         """Return sysinfo status"""
 
@@ -923,18 +1185,6 @@ class AsusRouter:
 
         return self._monitor_misc[KEY_TEMPERATURE]
 
-    async def async_get_temperature_labels(self) -> list[str]:
-        """Return list of temperature sensors"""
-
-        if not self._monitor_misc.ready:
-            await self.async_monitor_misc()
-
-        result = list()
-        for value in self._monitor_misc[KEY_TEMPERATURE]:
-            result.append(value)
-
-        return result
-
     async def async_get_vpn(self, use_cache: bool = True) -> dict[str, Any]:
         """Return VPN status"""
 
@@ -1051,6 +1301,10 @@ class AsusRouter:
 
         _LOGGER.debug(MSG_INFO["service"].format(service, arguments, result))
 
+        services = service.split(";")
+        if any(service in TRACK_SERVICES_LED for service in services):
+            await self.async_keep_state_led()
+
         if expect_modify:
             return converters.bool_from_any(result[AR_KEY_SERVICE_MODIFY])
         return True
@@ -1169,8 +1423,39 @@ class AsusRouter:
         await self.async_service_run(service=service, drop_connection=True)
         return True
 
+    async def async_get_firmware_update(self) -> dict[str, Any]:
+        """Check for firmware updates"""
+
+        result = dict()
+
+        # Check for updates
+        await self.async_command(
+            commands=None,
+            action_mode=AR_FIRMWARE_CHECK_COMMAND,
+            apply_to=AR_PATH["apply"],
+        )
+        # Get available firmware data
+        data = await self.async_load(AR_PATH["firmware"])
+
+        fw_current = self._identity.firmware
+        fw_new = parsers.firmware_string(data["webs_state_info"])
+
+        result["state"] = True if fw_current < fw_new else False
+        result["current"] = str(fw_current)
+        result["available"] = str(fw_new)
+
+        return result
+
     ### <-- SERVICES
 
+    async def async_keep_state_led(self) -> bool:
+        """Keep state of LEDs"""
+
+        # Only for Merlin firmware, so sysinfo should be present
+        if self._identity.sysinfo and self.led == False:
+            await self.async_service_led_set(True)
+            await self.async_service_led_set(False)
+
     @property
     def connected(self) -> bool:
         return self.connection.connected
