@@ -101,8 +101,6 @@ class Connection:
     ) -> dict[str, Any]:
         """Run command. Use the existing connection token, otherwise create new one"""
 
-        _LOGGER.debug(f"Started command {command}")
-
         if self._drop:
             return {}
 
@@ -134,7 +132,7 @@ class Connection:
                     else:
                         raise ex
             else:
-                _LOGGER.error(MSG_ERROR["command"].format(command, endpoint))
+                _LOGGER.error(MSG_ERROR["command"].format(endpoint))
                 return {}
 
     async def async_load(self, endpoint: str, retry: bool = False) -> str:
@@ -160,6 +158,10 @@ class Connection:
                 string_body = await r.text()
                 if "404 Not Found" in string_body:
                     raise AsusRouter404()
+        except (aiohttp.ClientError, asyncio.TimeoutError) as ex:
+            if not retry:
+                return self.async_load(endpoint, retry=True)
+            raise AsusRouterConnectionError(ex)
         except Exception as ex:
             raise ex
 
@@ -197,6 +199,8 @@ class Connection:
                 ssl=self._ssl,
             ) as r:
                 string_body = await r.text()
+                if string_body == str():
+                    return dict()
                 if "404 Not Found" in string_body:
                     raise AsusRouter404()
                 json_body = await r.json()
