@@ -21,6 +21,17 @@ AR_API = [
     "Httpd_AiHome_Ver",
 ]
 
+### VALUES
+AIMESH = "aimesh"
+CLIENTS = "clients"
+CONNECTION_TYPE = "connection_type"
+GUEST = "guest"
+IP = "ip"
+MAC = "mac"
+NODE = "node"
+ONBOARDING = "onboarding"
+RSSI = "rssi"
+
 ### CONSTANTS, DATA TYPES AND COMMON PARAMETERS
 CONST_BITSINBYTE = 8
 CONST_PERCENTS = 100
@@ -72,6 +83,11 @@ REGEX_VARIABLES = '([a-zA-Z0-9\_-]+)\s*=\s*"(.*?)"(?=;)'
 # VALUES TO IGNORE
 VALUES_TO_IGNORE = [str(), "None", "0.0.0.0"]
 
+# Services to recover LED state
+TRACK_SERVICES_LED = [
+    "restart_wireless",
+]
+
 
 ### ASUSWRT KEYS, MAPS AND VALUES
 AR_DEFAULT_CORES = [1]
@@ -95,6 +111,7 @@ AR_DEVICE_ATTRIBUTES_LIST: tuple[Key, ...] = (
     Key("wlConnectTime", "connected_since", time_from_delta),
     Key("curRx", "rx_speed", method=float_from_str),
     Key("curTx", "tx_speed", method=float_from_str),
+    Key("isGN", "guest", method=int_from_str),
 )
 AR_DEVICE_IDENTITY: tuple[Key, ...] = (
     Key("serial_no", "serial"),
@@ -123,6 +140,7 @@ AR_KEY_DEVICE_NAME_LIST = ["nickName", "name", "mac"]
 AR_KEY_DEVICEMAP = "devicemap"
 AR_KEY_DEVICES = "get_clientlist"
 AR_KEY_DEVICES_LIST = "maclist"
+AR_KEY_HEADER = "HEADER"
 AR_KEY_LED = "led_val"
 AR_KEY_LEDG_COUNT = "ledg_count"
 AR_KEY_LEDG_SCHEME = "ledg_scheme"
@@ -153,6 +171,10 @@ AR_KEY_OVPN_STATUS = (
     Key("pre-decompress bytes", "pre_decompress", method=int_from_str),
     Key("post-decompress bytes", "post_decompress", method=int_from_str),
 )
+AR_KEY_OVPN_STATUS_SERVER = (
+    Key("CLIENT_LIST", "client_list"),
+    Key("ROUTING_TABLE", "routing_table"),
+)
 AR_KEY_RAM = "memory_usage"
 AR_KEY_RAM_ITEM = "mem_{}"
 AR_KEY_RAM_LIST = [DATA_FREE, DATA_TOTAL, DATA_USED]
@@ -241,35 +263,54 @@ AR_MAP_TEMPERATURE: dict[str, list[str]] = {
     "2ghz": [
         'curr_coreTmp_2_raw="([0-9.]+)&deg;C',
         'curr_coreTmp_0_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_wl0_raw="([0-9.]+)&deg;C',
     ],
     "5ghz": [
         'curr_coreTmp_5_raw="([0-9.]+)&deg;C',
         'curr_coreTmp_1_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_wl1_raw="([0-9.]+)&deg;C',
     ],
     "5ghz2": [
         'curr_coreTmp_52_raw="([0-9.]+)&deg;C',
         'curr_coreTmp_2_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_wl2_raw="([0-9.]+)&deg;C',
+    ],
+    "6ghz": [
+        'curr_coreTmp_3_raw="([0-9.]+)&deg;C',
+        'curr_coreTmp_wl3_raw="([0-9.]+)&deg;C',
     ],
-    "6ghz": ['curr_coreTmp_3_raw="([0-9.]+)&deg;C'],
     "cpu": ['curr_cpuTemp="([0-9.]+)"', 'curr_coreTmp_cpu="([0-9.]+)"'],
 }
+AR_KEY_PARENTAL_CONTROL_STATE = "MULTIFILTER_ENABLE"
+AR_KEY_PARENTAL_CONTROL_NAME = "MULTIFILTER_DEVICENAME"
+AR_KEY_PARENTAL_CONTROL_MAC = "MULTIFILTER_MAC"
+AR_KEY_PARENTAL_CONTROL_TIMEMAP = "MULTIFILTER_MACFILTER_DAYTIME_V2"
 AR_MAP_PARENTAL_CONTROL: dict[str, str] = {
     "MULTIFILTER_ENABLE": "enable",
     "MULTIFILTER_DEVICENAME": "name",
     "MULTIFILTER_MAC": "mac",
     "MULTIFILTER_MACFILTER_DAYTIME_V2": "time",
 }
+AR_MAP_PARENTAL_CONTROL_STATE = {
+    "0": "disable",
+    "1": "time",
+    "2": "block",
+}
 AR_KEY_PARENTAL_CONTROL = Key(
     "MULTIFILTER_ALL", "parental_control", method=bool_from_any
 )
 
 AR_PATH = {
+    "apply": "apply.cgi",
     "command": "applyapp.cgi",
     "devicemap": "ajax_status.xml",
+    "devices_update": "update_clients.asp",
+    "firmware": "detect_firmware.asp",
     "get": "appGet.cgi",
     "ledg": "set_ledg.cgi",
     "login": "login.cgi",
     "logout": "Logout.asp",
+    "networkmap": "update_networkmapd.asp",
     "onboarding": "ajax_onboarding.asp",
     "ports": "ajax_ethernet_ports.asp",
     "rgb": "light_effect.html",
@@ -290,6 +331,7 @@ AR_VPN_STATUS = {
 AR_HOOK_TEMPLATE = "{}({});"
 AR_HOOK_DEVICES = "{}()".format(AR_KEY_DEVICES)
 
+AR_FIRMWARE_CHECK_COMMAND = "firmware_check"
 AR_SERVICE_COMMAND: dict[str, str] = {
     "force_restart": "restart_{}_force",
     "start": "start_{}",
@@ -331,7 +373,7 @@ ERROR_ZERO_DIVISION = "Zero division allert: {}"
 MSG_ERROR = {
     "authorisation": "Session is not authorised",
     "cert_missing": "CA certificate is missing",
-    "command": "Error sending a command {} to {}. Exception: {}",
+    "command": "Error sending a command to {}. Exception: {}",
     "credentials": "Wrong credentials",
     "disabled_control": "Device is connected in no-control mode. Sending commands is blocked",
     "disabled_monitor": "Device is connected in no-monitor mode. Sending hooks is blocked",
@@ -401,6 +443,63 @@ NVRAM_TEMPLATE = {
         Key("wl{}_wpa_gtk_rekey"),
         Key("wl{}_wpa_psk"),
     ],
+    "GWLAN": [
+        Key("wl{}_akm"),
+        Key("wl{}_ap_isolate"),
+        Key("wl{}_auth"),
+        Key("wl{}_auth_mode"),
+        Key("wl{}_auth_mode_x"),
+        Key("wl{}_bridge"),
+        Key("wl{}_bss_enabled", method=bool_from_any),
+        Key("wl{}_bss_maxassoc", method=int_from_str),
+        Key("wl{}_bw_dl", method=int_from_str),  # Bandwidth limit download
+        Key("wl{}_bw_enabled", method=bool_from_any),  # Bandwidth limit switch
+        Key("wl{}_bw_ul", method=int_from_str),  # Bandwidth limit upload
+        Key("wl{}_closed", method=bool_from_any),
+        Key("wl{}_crypto"),
+        Key("wl{}_expire", method=int_from_str),  # Expire time in s
+        Key("wl{}_expire_tmp", method=int_from_str),  # Expire time left in s
+        Key("wl{}_gn_wbl_enable"),
+        Key("wl{}_gn_wbl_rule"),
+        Key("wl{}_gn_wbl_type"),
+        Key("wl{}_hwaddr"),  # MAC address
+        Key("wl{}_ifname"),  # Interface name
+        Key("wl{}_infra"),
+        Key("wl{}_key"),
+        Key("wl{}_key1"),
+        Key("wl{}_key2"),
+        Key("wl{}_key3"),
+        Key("wl{}_key4"),
+        Key("wl{}_lanaccess", method=bool_from_any),  # LAN access
+        Key("wl{}_maclist"),
+        Key("wl{}_macmode"),
+        Key("wl{}_maxassoc", method=int_from_str),
+        Key("wl{}_mbss"),
+        Key("wl{}_mfp"),
+        Key("wl{}_mode"),
+        Key("wl{}_net_reauth", method=int_from_str),
+        Key("wl{}_preauth"),
+        Key("wl{}_radio", method=bool_from_any),
+        Key("wl{}_radius_ipaddr"),
+        Key("wl{}_radius_key"),
+        Key("wl{}_radius_port", method=int_from_str),
+        Key("wl{}_sae_anti_clog_threshold"),
+        Key("wl{}_sae_groups"),
+        Key("wl{}_sae_sync"),
+        Key("wl{}_ssid"),  # SSID
+        Key("wl{}_sta_retry_time"),
+        Key("wl{}_sync_node", method=bool_from_any),  # Sync AiMesh nodes
+        Key("wl{}_unit"),  # GWLAN unit id
+        Key("wl{}_wep", method=bool_from_any),
+        Key("wl{}_wep_x", method=bool_from_any),
+        Key("wl{}_wfi_enable", method=bool_from_any),
+        Key("wl{}_wfi_pinmode"),
+        Key("wl{}_wme"),
+        Key("wl{}_wme_bss_disable", method=bool_from_any),
+        Key("wl{}_wpa_gtk_rekey"),
+        Key("wl{}_wpa_psk"),  # Password
+        Key("wl{}_wps_mode"),
+    ],
 }
 
 NVRAM_LIST = {
