@@ -4,9 +4,12 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 from datetime import datetime
+from typing import Any
 
 from asusrouter.util.converters import none_or_str
 
+DEFAULT_PARENTAL_CONTROL_TIMEMAP = "W03E21000700<W04122000800"
+
 
 @dataclass
 class ConnectedDevice:
@@ -22,6 +25,7 @@ class ConnectedDevice:
     internet_mode: bool | None = None
 
     connection_type: str | None = None
+    node: str | None = None
 
     # WLAN only values
     online: bool | None = None
@@ -30,6 +34,8 @@ class ConnectedDevice:
     rx_speed: float | None = None
     tx_speed: float | None = None
 
+    guest: int = 0
+
 
 @dataclass
 class AsusDevice:
@@ -39,9 +45,7 @@ class AsusDevice:
     mac: str | None = None
     model: str | None = None
     brand: str = "ASUSTek"
-    fw_major: str | None = None
-    fw_minor: str | None = None
-    fw_build: str | None = None
+    firmware: Firmware | None = None
     services: str | None = None
     sysinfo: bool = False
     led: bool = False
@@ -49,9 +53,34 @@ class AsusDevice:
     aura: bool = False
     vpn_status: bool = False
     onboarding: bool = False
+    update_networkmapd: bool = False
 
-    def firmware(self) -> str:
-        return "{}.{}_{}".format(self.fw_major, self.fw_minor, self.fw_build)
+
+@dataclass
+class AiMeshDevice:
+    """AiMesh device class"""
+
+    # Status
+    status: bool = False
+
+    alias: str | None = None
+    model: str | None = None
+    product_id: str | None = None
+    ip: str | None = None
+
+    fw: str | None = None
+    fw_new: str | None = None
+
+    mac: str | None = None
+
+    # Access point: ap2g, ap5g, ap5g1, ap6g, apdwb
+    ap: dict[str, Any] | None = None
+    # Parent AiMesh: pap2g, rssi2g, pap2g_ssid, pap5g, rssi5g, pap5g_ssid, pap6g, rssi6g, pap6g_ssid
+    parent: dict[str, Any] | None = None
+    # Node state
+    state: int | None = None
+    level: int | None = None
+    config: dict[str, Any] | None = None
 
 
 @dataclass
@@ -111,6 +140,7 @@ class Monitor(dict):
     active: bool = False
     time: datetime | None = None
     ready: bool = False
+    enabled: bool = True
 
     def start(self) -> None:
         """Set to active"""
@@ -137,3 +167,43 @@ class Monitor(dict):
 
         self.stop()
         self.ready = False
+
+
+@dataclass
+class Firmware:
+    """Firmware class"""
+
+    major: str = "3.0.0.4"
+    minor: int | None = None
+    build: int | None = None
+    build_more: int | str | None = None
+
+    def __lt__(self, other: Firmware) -> bool:
+        """Define less-than"""
+
+        if self.minor and other.minor and self.minor < other.minor:
+            return True
+        if self.build and other.build and self.build < other.build:
+            return True
+        if (
+            type(self.build_more) == int
+            and type(self.build_more) == type(other.build_more)
+            and self.build_more < other.build_more
+        ):
+            return True
+        return False
+
+    def __str__(self) -> str:
+        """Define string"""
+
+        return f"{self.major}.{self.minor}.{self.build}_{self.build_more}"
+
+
+@dataclass(frozen=True)
+class FilterDevice:
+    """Device filter class"""
+
+    mac: str | None = None
+    name: str | None = None
+    state: int | None = None
+    timemap: str = DEFAULT_PARENTAL_CONTROL_TIMEMAP
