@@ -20,19 +20,28 @@ from asusrouter.const import (
     AR_DEFAULT_CORES_RANGE,
     AR_DEFAULT_LEDG,
     AR_DEFAULT_OVPN_CLIENTS,
+    AR_DEFAULT_OVPN_SERVERS,
     AR_DEVICE_ATTRIBUTES_LIST,
     AR_KEY_CPU_ITEM,
     AR_KEY_CPU_LIST,
     AR_KEY_DEVICEMAP,
+    AR_KEY_HEADER,
     AR_KEY_NETWORK_GROUPS,
     AR_KEY_NETWORK_ITEM,
     AR_KEY_OVPN,
     AR_KEY_OVPN_STATUS,
+    AR_KEY_OVPN_STATUS_SERVER,
     AR_KEY_PARENTAL_CONTROL,
+    AR_KEY_PARENTAL_CONTROL_MAC,
+    AR_KEY_PARENTAL_CONTROL_NAME,
+    AR_KEY_PARENTAL_CONTROL_STATE,
+    AR_KEY_PARENTAL_CONTROL_TIMEMAP,
     AR_KEY_RAM_ITEM,
     AR_KEY_RAM_LIST,
     AR_KEY_VPN_CLIENT,
+    AR_KEY_VPN_SERVER,
     AR_KEY_WAN_STATE,
+    AR_MAP_PARENTAL_CONTROL_STATE,
     AR_MAP_RGB,
     AR_MAP_SYSINFO,
     AR_MAP_TEMPERATURE,
@@ -56,9 +65,9 @@ from asusrouter.const import (
     REGEX_VARIABLES,
     VALUES_TO_IGNORE,
 )
-from asusrouter.dataclass import ConnectedDevice
+from asusrouter.dataclass import AiMeshDevice, ConnectedDevice, FilterDevice, Firmware
 from asusrouter.error import AsusRouterNotImplementedError, AsusRouterValueError
-from asusrouter.util import calculators
+from asusrouter.util import calculators, converters
 
 
 def cpu_cores(raw: dict[str, Any] | None = None) -> list[int]:
@@ -406,6 +415,7 @@ def sysinfo(raw: str) -> dict[str, Any]:
 def onboarding(raw: str) -> dict[str, Any]:
     """Onboarding parser"""
 
+    raw = raw.replace("\ufeff", "")
     raw = raw.replace("=", '":')
     raw = raw.replace(";", ',"')
     raw = raw.replace("[0]", "")
@@ -417,6 +427,56 @@ def onboarding(raw: str) -> dict[str, Any]:
     return data
 
 
+def aimesh_node(raw: dict[str, Any]) -> AiMeshDevice:
+    """AiMesh node parser"""
+
+    AP = {
+        "2g": "2ghz",
+        "5g": "5ghz",
+        "5g1": "5ghz2",
+        "6g": "6ghz",
+        "dwb": "dwb",
+    }
+
+    FREQUENCIES = [
+        "2g",
+        "5g",
+        "6g",
+    ]
+
+    ap = dict()
+    for el in AP:
+        if f"ap{el}" in raw and raw[f"ap{el}"] is not str():
+            ap[AP[el]] = raw[f"ap{el}"]
+
+    parent = dict()
+    for el in FREQUENCIES:
+        if f"pap{el}" in raw and raw[f"pap{el}"] is not str():
+            parent["connection"] = AP[el]
+            parent["mac"] = raw[f"pap{el}"]
+            parent["rssi"] = converters.none_or_str(raw.get(f"rssi{el}"))
+            parent["ssid"] = converters.none_or_str(raw.get(f"pap{el}_ssid"))
+
+    level = converters.int_from_str(raw.get("level", "0"))
+    state = "router" if level == 0 else "node"
+
+    return AiMeshDevice(
+        status=converters.bool_from_any(raw.get("online", 0)),
+        alias=raw.get("alias", None),
+        model=raw.get("ui_model_name", raw.get("model_name", None)),
+        product_id=raw.get("product_id"),
+        ip=raw.get("ip"),
+        fw=raw.get("fwver", None),
+        fw_new=converters.none_or_str(raw.get("newfwver")),
+        mac=raw.get("mac", None),
+        ap=ap,
+        parent=parent,
+        state=state,
+        level=level,
+        config=raw.get("config"),
+    )
+
+
 def parental_control(raw: dict[str, Any]) -> dict[str, Any]:
     """Parental control parser"""
 
@@ -425,6 +485,12 @@ def parental_control(raw: dict[str, Any]) -> dict[str, Any]:
     result[AR_KEY_PARENTAL_CONTROL.get()] = AR_KEY_PARENTAL_CONTROL.method(
         raw[AR_KEY_PARENTAL_CONTROL.value]
     )
+
+    # If no rules are set
+    if raw.get(AR_KEY_PARENTAL_CONTROL_MAC) == raw.get(AR_KEY_PARENTAL_CONTROL_STATE):
+        result["list"] = dict()
+        return result
+
     list = dict()
 
     for el in raw:
@@ -433,12 +499,17 @@ def parental_control(raw: dict[str, Any]) -> dict[str, Any]:
     size = len(raw["MULTIFILTER_MAC"])
 
     for i in range(0, size):
-        data = {
-            "access": raw["MULTIFILTER_ENABLE"][i],
-            "name": raw["MULTIFILTER_DEVICENAME"][i],
-            "time": raw["MULTIFILTER_MACFILTER_DAYTIME_V2"][i],
-        }
-        list[raw["MULTIFILTER_MAC"][i]] = data.copy()
+        device = FilterDevice(
+            mac=raw[AR_KEY_PARENTAL_CONTROL_MAC][i],
+            name=raw[AR_KEY_PARENTAL_CONTROL_NAME][i],
+            state=AR_MAP_PARENTAL_CONTROL_STATE.get(
+                raw[AR_KEY_PARENTAL_CONTROL_STATE][i], None
+            ),
+            timemap=raw[AR_KEY_PARENTAL_CONTROL_TIMEMAP][i]
+            if i < len(raw[AR_KEY_PARENTAL_CONTROL_TIMEMAP])
+            else "",
+        )
+        list[raw[AR_KEY_PARENTAL_CONTROL_MAC][i]] = device
 
     result["list"] = list.copy()
 
@@ -450,6 +521,8 @@ def pseudo_json(text: str, page: str) -> dict[str, Any]:
 
     if page == AR_PATH["vpn"]:
         return vpn_status(text)
+    if page == AR_PATH["firmware"]:
+        return firmware(text)
 
     data = re.sub("\s+", "", text)
 
@@ -507,6 +580,55 @@ def ovpn_client_status(raw: str) -> dict[str, Any]:
     return values
 
 
+def ovpn_server_status(raw: str) -> dict[str, Any]:
+    """OpenVPN server status parser"""
+
+    if type(raw) != str:
+        raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
+    raw = raw.strip()
+    if raw == str() or raw == "None":
+        return {}
+
+    values = dict()
+
+    for key in AR_KEY_OVPN_STATUS_SERVER:
+        if key.value in raw:
+            flag = False
+            # Find headers
+            header = re.search(f"{AR_KEY_HEADER},{key.value},(.*?)(?=>)", raw)
+            if header:
+                keys_raw = header[1].split(",")
+                keys = list()
+                for el in keys_raw:
+                    keys.append(converters.to_snake_case(el))
+                flag = True
+
+            # Hide header
+            raw = raw.replace(f"{AR_KEY_HEADER},{key.value},", "USED-")
+
+            values[key.get()] = list()
+            # Find values
+            while value := re.search(f"{key.value},(.*?)(?=>)", raw):
+                if not value:
+                    break
+                if flag:
+                    cut = value[1].split(",")
+                    i = 0
+                    set = dict()
+                    while i < len(cut) and i < len(keys):
+                        set[keys[i]] = cut[i]
+                        i += 1
+                    values[key.get()].append(set.copy())
+                else:
+                    if key.method:
+                        values[key.get()].append(key.method(value[1]))
+                    else:
+                        values[key.get()].append(value[1])
+                raw = raw.replace(f"{key.value},{value[1]}", "USED-")
+
+    return values
+
+
 # var = "value";
 def variables(raw: str) -> dict[str, Any]:
     """Variables parser"""
@@ -528,6 +650,56 @@ def variables(raw: str) -> dict[str, Any]:
     return values
 
 
+def firmware(raw: str) -> dict[str, Any]:
+    """Firmware parser"""
+
+    if type(raw) != str:
+        raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
+    if raw.strip() == str():
+        return {}
+
+    values = variables(raw.replace("'", '"'))
+
+    return values
+
+
+def firmware_string(raw: str) -> Firmware:
+    """Firmware string parser"""
+
+    if type(raw) != str:
+        raise AsusRouterValueError(ERROR_VALUE_TYPE.format(raw, type(raw)))
+
+    string = re.match(
+        "^(3.?0.?0.?4)?[_.]?([0-9]{3})[_.]?([0-9]+)[_.-]?([a-zA-Z0-9-_]+)?$", raw
+    )
+    if not string:
+        _LOGGER.warning(
+            f"Firmware version cannot be parser. Please report this. The original FW string is: {raw}"
+        )
+        return Firmware()
+
+    major = string[1]
+    if major and not "." in major and len(major) == 4:
+        major = major[0] + "." + major[1] + "." + major[2] + "." + major[3]
+
+    minor = int(string[2])
+    build = int(string[3])
+    if string[4] and string[4].isdigit():
+        build_more = int(string[4])
+    else:
+        build_more = string[4]
+
+    fw = Firmware(
+        minor=minor,
+        build=build,
+        build_more=build_more,
+    )
+    if major:
+        fw.major = major
+
+    return fw
+
+
 def vpn_status(raw: str) -> dict[str, Any]:
     """VPN status parser"""
 
@@ -540,6 +712,7 @@ def vpn_status(raw: str) -> dict[str, Any]:
 
     values = variables(raw)
 
+    # Clients
     for num in range(1, AR_DEFAULT_OVPN_CLIENTS + 1):
         key = AR_KEY_OVPN.format(AR_KEY_VPN_CLIENT, num, PARAM_STATUS)
         if not key in values:
@@ -562,4 +735,27 @@ def vpn_status(raw: str) -> dict[str, Any]:
         if key in values and values[key] not in VALUES_TO_IGNORE:
             vpns[vpn_id][PARAM_RIP] = values[key]
 
+    # Servers
+    for num in range(1, AR_DEFAULT_OVPN_SERVERS + 1):
+        key = AR_KEY_OVPN.format(AR_KEY_VPN_SERVER, num, PARAM_STATUS)
+        if not key in values:
+            break
+
+        vpn_id = AR_KEY_VPN_SERVER + str(num)
+
+        if values[key] not in VALUES_TO_IGNORE:
+            vpns[vpn_id] = ovpn_server_status(values[key])
+            vpns[vpn_id][PARAM_STATE] = True
+        else:
+            vpns[vpn_id] = dict()
+            vpns[vpn_id][PARAM_STATE] = False
+
+        key = AR_KEY_OVPN.format(AR_KEY_VPN_SERVER, num, PARAM_IP)
+        if key in values and values[key] not in VALUES_TO_IGNORE:
+            vpns[vpn_id][PARAM_IP] = values[key]
+
+        key = AR_KEY_OVPN.format(AR_KEY_VPN_SERVER, num, PARAM_RIP)
+        if key in values and values[key] not in VALUES_TO_IGNORE:
+            vpns[vpn_id][PARAM_RIP] = values[key]
+
     return vpns
